%% Multilayer Model for Curvature, Force, and Work: GUIDED SETUP
% Created by Aleksandar Nacev
% 20 April, 2012
% Calls 'PlotMultilayerModel.m' and 'DesignBasic.m' and 'DesignBasic.fig'

clear all
clc

% This code creates a GUI which allows a researcher to design their own
% custom actuator either from scratch or starting with 7 preset examples.
% The actuator can be in designed in dimensional units or
% non-dimensionalized. 3-dimensional, 2-dimensional, and single value
% display information is available based upon the specific variations of
% values. All units used are in standard SI. There are many more specific
% edits that can be adjusted for a particular bender, but this Build GUI
% addresses only the main more typical use cases. 

% Additional Notes:
% The individual layer widths are set to the device width. This can be
% edited by specifically using the 'PlotMultilayerModel.m' and
% 'MulitlayerModel.m' functions appropriately. The strains are defined as a
% percentage in the GUI but are used in decimal form in the
% 'MultilayerModel.m'. 
% 
% --------------------------------------------------- %



% Display the Welcome dialog box
welcomes = msgbox({'Welcome to a guided set-up of a multilayer bending model.';...
    ' ';...
    'This GUI is designed to help you create your own multilayer bender using either preset configurations or a custom made configuration.';...
    ' ';...
    'This GUI is able to display either a 3-dimensional plot, 2-dimensional plot, or single value information (displayed in the command window). Enjoy.';...
    ' ';...
    'If you use this program to design a device, you should credit this publication ("Design of Multi-Layer Bending Electroactive Polymer Actuators" by Bavani Balakrisnan, Alek Nacev, and Elisabeth Smela).';...
    ' ';...
    'NOTE: All units shown are standard SI units.';...
    ' '},'MULTILAYER MODELING GUIDED SET-UP');
uiwait(welcomes);

% BEGIN MODEL CREATION
        ENDDESIGN =  0; % Tag for ending design creation
        while ENDDESIGN == 0
            
%% CHOOSE BETWEEN CUSTOM SET-UP OR 4 SAMPLE CASES
CASE_optionlist = {'Bi-Layer Model (2 layers, layer 1 changes volume)';...
    'Tri-Layer Model (3 layers, layer 2 changes volume)';...
    'Trilayer Plus Passive External Layer (4 layers total, layer 2 changes volume)';...
    '2-Bilayer Stack (4 layers, layers 2 and 4 change volume)';...
    'IPMC: Face-to-Face Bilayers (4 layers:  layers 2 and 3 change volume)';...
    'CP: Face-to-Face Bilayer (4 layers:  layers 1 and 4 change volume)';...
    'Five Layer Device (layers 2 and 4 change volume)';...
    'CUSTOM DEVICE';...
    'Load File'};

[CASESelect,ok] = listdlg('Name','BASIC SETUP: MODES',...
    'PromptString','Select an operation mode:',...
    'SelectionMode','single',...
    'OKString','CONTINUE',...
    'ListSize',[400 150],...
    'ListString',CASE_optionlist);
if ok == 0
    return
end

%----- % SELECTIONS FROM CASE LIST % -----%
switch CASESelect
    case 9 % Load a file
        uiopen('LOAD')
        
        % Convert Files to GUI Interface
        LV =  options.LV;
        D = options.D;
        INDV = find(LV>0,1,'first');
        LVary = [LayerE(INDV,:);LayerT(INDV,:)];
        LStat = [LayerE(:,1)' ; LayerT(:,1)'];
        DeviceW = LayerW(1);
        
        clearvars -except NLayers D LVary LStat LayerS DeviceL DeviceW LV options LayerP
                
        % RUN GUI
        [LVary LStat LayerS DeviceL DeviceW LV ENDDESIGN ND DIM FIGV] = DesignBasic(NLayers, D, LVary, LStat, LayerS, DeviceL, DeviceW, LV, LayerP);
                
        
    case 8 % CUSTOM ACTUATOR
        

            
            % BASIC Setup Questions: Choose between Two Modes
            MODE_optionlist = {'Non-Dimensional Plot';'Dimensional Plot'};
            [MODESelect,ok] = listdlg('Name','BASIC SETUP: MODES',...
                'PromptString','Select an operation mode:',...
                'SelectionMode','single',...
                'OKString','CONTINUE',...
                'ListSize',[250 40],...
                'ListString',MODE_optionlist);
            if ok == 0
                return
            end
            D = MODESelect - 1;
            
            % Choose How Many Layers are in Bender
            
            % windows sizing options
            woptions.Resize = 'on';
            woptions.WindowStyle = 'normal';
            woptions.Interpreter = 'tex';
            
            % prompt user on how many layers are in device
            prompt = {'How Many Layers in Device?'};
            name = 'NUMBER OF LAYERS';
            numlines = 1;
            defaultanswer = {'3'};
            answer = inputdlg(prompt,name,numlines,defaultanswer,woptions);
            NLayers = str2double(char(answer(1))); % convert string to numerical answer
            LayerP = ones(NLayers,1)*0.5;

            options.n = 2;
            options.ttag = char(strcat({'CUSTOM MODEL ('},num2str(NLayers),{' LAYERS)'}));
            
            % RUN GUI
            [LVary LStat LayerS DeviceL DeviceW LV ENDDESIGN ND DIM FIGV] = DesignBasic(NLayers, D, LayerP);
            
            % Add miscellaneous tags
            options.ND = ND;

    case {1,2,3,4,5,6,7}
        
        % BASIC Setup Questions: Choose between Two Modes
        MODE_optionlist = {'Non-Dimensional Plot';'Dimensional Plot'};
        [MODESelect,ok] = listdlg('Name','BASIC SETUP: MODES',...
            'PromptString','Select an operation mode:',...
            'SelectionMode','single',...
            'OKString','CONTINUE',...
            'ListSize',[250 40],...
            'ListString',MODE_optionlist);
        if ok == 0
            return
        end
        D = MODESelect - 1;
        
        % REACHABLE Boundaries For All Examples %
        temin = 100e-10;    % electrode min thickness
        temax = 10e-6;      % electrode max thickness
        Eemin = 1e3;        % electrode min modulus
        Eemax = 100e9;      % electrode max modulus
        tpmin = 1e-6;       % polymer min thickness
        tpmax = 1e-3;       % polymer max thickness
        Epmin = 1e4;        % polymer min modulus
        Epmax = 10e6;       % polymer max modulus
        
        Ermin = Eemin/Epmax;
        Ermax = Eemax/Epmin;
        trmin = temin/tpmax;
        trmax = temax/tpmin;
        
        LVary = [Ermin Ermax; trmin trmax];
        LStat = [1; 1];
        DeviceW = 1;
        DeviceL= 1;
        
        switch CASESelect
            case 1
                %---- % BI-LAYER MODEL % ----%
                NLayers = 2;
                % PARAMETERS
                LayerS = [0 ; -0.01];
                LV = [1 ; 0];
                LStat = repmat(LStat,1,NLayers);
                LayerP = ones(NLayers,1)*0.5;

                options.n = 2;
                options.ttag = 'BI-LAYER MODEL';
                
                % RUN GUI
                [LVary LStat LayerS DeviceL DeviceW LV ENDDESIGN ND DIM FIGV] = DesignBasic(NLayers, D, LVary, LStat, LayerS, DeviceL, DeviceW, LV, LayerP);
                
                options.ND = ND;
                
            case 2
                %---- % TRI-LAYER MODEL % ----%
                NLayers = 3;
                % PARAMETERS
                LayerS = [0 ; -0.01; 0];
                LV = [1 ; 0 ; 0];
                LStat = repmat(LStat,1,NLayers);
                LStat(2,3) = 1e-4;
                LayerP = ones(NLayers,1)*0.5;

                options.n = 2;
                options.ttag = 'TRI-LAYER MODEL';
                
                % RUN GUI
                [LVary LStat LayerS DeviceL DeviceW LV ENDDESIGN ND DIM FIGV] = DesignBasic(NLayers, D, LVary, LStat, LayerS, DeviceL, DeviceW, LV, LayerP);
                
                options.ND = ND;
            case 3
                %---- % 2-Bilayer Stack with Passive External Layer % ----%
                NLayers = 4;
                
                % PARAMETERS
                LayerS = [0 ; 0.01; 0; 0];
                LV = [1 ; 0 ; 1; 0];
                LStat = [1; 0.5];
                LStat = repmat(LStat,1,NLayers);
                LayerP = ones(NLayers,1)*0.5;

                options.n = 2;
                options.ttag = '2-Bilayer Stack with Passive External Layer';
                
                % RUN GUI
                [LVary LStat LayerS DeviceL DeviceW LV ENDDESIGN ND DIM FIGV] = DesignBasic(NLayers, D, LVary, LStat, LayerS, DeviceL, DeviceW, LV, LayerP);
                
                options.ND = ND;
                
            case 4
                %---- % 2-Bilayer Stack % ----%
                NLayers = 4;
                
                % PARAMETERS
                LayerS = [0 ; -0.01; 0; -0.01];
                LV = [1 ; 0 ; 1; 0];
                LStat = [1; 0.5];
                LStat = repmat(LStat,1,NLayers);
                LayerP = ones(NLayers,1)*0.5;

                options.n = 2;
                options.ttag = '2-Bilayer Stack';
                
                % RUN GUI
                [LVary LStat LayerS DeviceL DeviceW LV ENDDESIGN ND DIM FIGV] = DesignBasic(NLayers, D, LVary, LStat, LayerS, DeviceL, DeviceW, LV, LayerP);
                
                options.ND = ND;
                
            case 5
                %---- % IMPC: Face-to-face Bilayer % ----%
                NLayers = 4;
                
                % PARAMETERS
                LayerS = [0 ; 0.01; -0.01; 0];
                LV = [1 ; 0 ; 0; 1];
                LStat = [1; 0.5];
                LStat = repmat(LStat,1,NLayers);
                LayerP = ones(NLayers,1)*0.5;

                % RUN GUI
                [LVary LStat LayerS DeviceL DeviceW LV ENDDESIGN ND DIM FIGV] = DesignBasic(NLayers, D, LVary, LStat, LayerS, DeviceL, DeviceW, LV, LayerP);
                
                options.n = 2;
                options.ND = ND;
                options.ttag = 'IMPC: Face-to-face Bilayer';
                
            case 6
                %---- % Face-to-face Bilayer with EAP on Exterior % ----%
                NLayers = 4;
                
                % PARAMETERS
                LayerS = [0.01; 0; 0; -0.01];
                LV = [0 ; 1 ; 1; 0];
                LStat = [1; 0.5];
                LStat = repmat(LStat,1,NLayers);
                LayerP = ones(NLayers,1)*0.5;

                % RUN GUI
                [LVary LStat LayerS DeviceL DeviceW LV ENDDESIGN ND DIM FIGV] = DesignBasic(NLayers, D, LVary, LStat, LayerS, DeviceL, DeviceW, LV, LayerP);
                
                options.n = 2;
                options.ND = ND;
                options.ttag = 'Face-to-face Bilayer with EAP on Exterior';
                
            case 7
                %---- % Five Layer Device % ----%
                NLayers = 5;
                
                % PARAMETERS
                LayerS = [0; 0.01; 0; 0; 0];
                LV = [1; 0 ; 1 ; 0; 1];
                LStat = [1; 0.5];
                LStat = repmat(LStat,1,NLayers);
                LayerP = ones(NLayers,1)*0.5;

                % RUN GUI
                [LVary LStat LayerS DeviceL DeviceW LV ENDDESIGN ND DIM FIGV] = DesignBasic(NLayers, D, LVary, LStat, LayerS, DeviceL, DeviceW, LV, LayerP);
                
                options.n = 2;
                options.ND = ND;
                options.ttag = 'Five Layer Device';
                
        end
end
        end
        

%% PLOT MULTILAYER MODELS
LayerE = zeros(NLayers,2);
LayerT = zeros(NLayers,2);
LayerW = zeros(NLayers,1);

% Create matrices from output data from GUI to pass into
% 'PlotMultilayerModel.m'
for i = 1:NLayers
    if LV(i) == 1
        LayerE(i,:) = LVary(1,:);
        LayerT(i,:) = LVary(2,:);
    else
        LayerE(i,:) = [LStat(1,i) LStat(1,i)];
        LayerT(i,:) = [LStat(2,i) LStat(2,i)];
    end
    LayerW(i) = DeviceW;
end

options.LV = LV;
options.dimen = DIM;
options.D = D;

% Plot Bending Figure
scrsz = get(0,'ScreenSize');
FigureBender = figure('Position',[10 scrsz(4)/2-100 scrsz(3)/2 scrsz(4)/2],'Name','Bending Device');
copyobj(get(FIGV,'Children'),gca);
axis off;
title('Bending Device','FontSize',16)
close(get(FIGV,'Parent'));

clearvars -except LayerE LayerT LayerS LayerP LayerW DeviceL NLayers options FigureBender

PlotMultilayerModel(LayerE,LayerT,LayerS,LayerP,LayerW,DeviceL,NLayers,options)

