%% Multilayer Model
% Created by Aleksandar Nacev
% August 15, 2011

% COMMENTS FROM CALL FUNCTION:
    % NOTE on usuage of function
    % MultiLayerModel will take a arbitrary number of layers designated by the
    % thickness, width, young's modulus and strain vectors given to it. The
    % young's modulus is automatically scaled assuming an incompressible
    % material. If a compressible material is required, the MultiLayerModel
    % must be edited. The width only has an effect if it is not the same for
    % all layers. The strain is the same for the tri-layer model, where you
    % input a strain that is relative to the first layer.

    % layers are defined from the bottom upwards

function Kappa = MultiLayerModel(t,width,sE,ep,v)

% Determine of Layers
num = length(t);

% Set Geometric Variables
a = t.*width; %Cross-sectional Area
I = (1/12)*width.*t.*t.*t; % Moment of Inertia
E = sE./(1-v.*v); % Convert E for compressibility factor

%% Matrix Formation for Solving

% A Matrix / D Matrix / B Matrix / C Matrix

A = zeros(num,num); % initialize A matrix
lD = 0; % initialize the left hand side of D
rD = zeros(1,num); % initialize right hand side of D matrix
B = zeros(num,1); % initialize B matrix
C = zeros(num,1); % initialize C matrix - STRAIN from Elast

for i = 1:num-1
    
    % A matrix
    A(i,i) = 1/(a(i)*E(i)); % use a vector and E vector
    A(i,i+1) = -1/(a(i+1)*E(i+1)); % second term in A matrix
    
    % lD matrix
    lD = lD + E(i)*I(i); % left hand side of D (inverted)
    
    % rD matrix
    if i > 1
        rD(1,i) = sum(t(1:i-1))+0.5*t(i);
    else
        rD(1,i) = 0.5*t(i);
    end
    
    % B matrix
    B(i,1) = t(i)+t(i+1);
    
    % C matrix
    C(i,1) = ep(i+1)-ep(i);
    
end

% Last Components of Matrices

% A
A(num,:) = ones(1,num); % set last row to ones

% rD
rD(1,num) = sum(t(1:num-1))+0.5*t(num);

% lD
lD = lD + E(num)*I(num);

% Make Entire D vector now
D = (1/lD)*rD;

% B and C do not need another component since it was initialized with a
% 'zero' matrix - 0's in the last row already

%% Solve For Kappa

Kappa = (2*D/A*C)/(2-D/A*B); % Solve for Kappa

end
