%% Call the Function "MultiLayerModel" to produce a graph for varied parameters
% Created by Aleksandar Nacev on 17 Apr 2012
% For use with 'BuildMultilayerModel.m'
% Calls 'MultiLayerModel.m' and 'BlockingForce_MultiLayerModel.m'

% INPUTS ARE AS FOLLOWS:
% LayerE  = a column vector with the min and max of each layer's Young's Modulus
% LayerT  = a column vector with the min and max of each layer's height
% LayerS  = a column vector with each layer's strain in decimal form
% LayerP  = a column vector with each layer's Poisson's ratio
% LayerW  = a column vector with each layer's width
% DeviceL = the length of the device
% NLayers = the number of layers
% options = has six formatting options:
% options.ND:    a tag to choose whether to scale the plots by the amount
%                show in the paper
% options.n:     the amount of resoltion to the curves
% options.ttag:  the title tag that is added to each plot
% options.dimen: a tag for whether the plot will be a single point (1),
%                2D (2) or 3D (3)
% options.D:     a tag if the plot is non-dimensional (0) or not (1)
% options.LV:    a vector specifying which layers are varying



function [] = PlotMultilayerModel(LayerE,LayerT,LayerS,LayerP,LayerW,DeviceL,NLayers,options)

% Adjust the parameters accordingly in the required sections

% Switching tag for whether to scale curves to create plots on the
% same axis as shown in the paper


% These lines allow the program to run with a saved model.
if isempty(options.dimen)
    options.dimen = options.DIM
end

switch options.ND
    case 1
        scaleK = 0.015;
        scaleF = 0.11;
        scaleW = 1.8739e-5;
    case 0
        scaleK = 1;
        scaleF = 1;
        scaleW = 1;
end

% Two parameters can be varied: two of the Young's moduli, two of
% the thicknesses, or one of each.  One could also vary the strain,
% but bending is linear with strain.

a = 20*options.n; % number of plot points for 1st variable
b = 20*options.n; % number of plot points for 2nd variable

% Young's moduli
EMatrix = zeros(NLayers,a);
for i = 1:NLayers
    EMatrix(i,:) = logspace(log10(LayerE(i,1)),log10(LayerE(i,2)),a);
end

% Thicknesses
TMatrix = zeros(NLayers,b);
for i = 1:NLayers
    TMatrix(i,:) = logspace(log10(LayerT(i,1)),log10(LayerT(i,2)),b);
end

% Loop over the two variables to determine Kappa, Force, and Work
for i = 1:a
    for j = 1:b
        % i for the first variable, j for the 2nd.
        Kappa(i,j) = MultiLayerModel(TMatrix(:,j)', LayerW', EMatrix(:,i)', LayerS', LayerP');
        Force(i,j) = BlockingForce_MultiLayerModel(TMatrix(:,j)', LayerW', EMatrix(:,i)', Kappa(i,j), DeviceL, LayerP');
        Deflection(i,j)= 0.5*DeviceL*DeviceL*Kappa(i,j);
        Work(i,j)= 0.25*Force(i,j)*Deflection(i,j);
    end
end

% For Plotting Figures in Correct Location
scrsz = get(0,'ScreenSize');

switch options.dimen
    case 1 % SINGLE POINT
        switch options.D
            case 0
                disp('Your curvature (Kappa) is:')
                disp(Kappa(1,1))
                disp('Your blocked force is:')
                disp(Force(1,1))
                disp('Your work is:')
                disp(Work(1,1))
                return
            case 1
                disp('Your curvature (Kappa) in [1/m] is:')
                disp(Kappa(1,1))
                disp('Your blocked force in [N] is:')
                disp(Force(1,1))
                disp('Your work in [J] is:')
                disp(Work(1,1))
                return
        end
        
    case 3 % 3D- PLOT
        % Determine which variable was changing to use as the axis for the plots
        Eind = find(options.LV>0,1,'first');
        Tind = find(options.LV>0,1,'first');
        Ev = EMatrix(Eind,:);
        Tv = TMatrix(Tind,:);
        layerstring = {'Layers '};
        for i = 1:NLayers
            if options.LV(i) == 1
                if i == Eind
                    layerstring = char(strcat(layerstring,{' '},num2str(i)));
                else
                    layerstring = char(strcat(layerstring,{', '},num2str(i)));
                end
            end
        end
       
        
    case 2 % 2D-PLOT
        if sum(LayerE(:,2)-LayerE(:,1))>0
            XIND = find(EMatrix(:,2)-EMatrix(:,1)>0,1,'first');
            XAXIS = EMatrix(XIND,:);
            KAXIS = Kappa(:,1)'/scaleK;
            FAXIS = Force(:,1)'/scaleF;
            WAXIS = Work(:,1)'/scaleW;
            layerstring = {'Layers '};
            for i = 1:NLayers
                if options.LV(i) == 1
                    if i == XIND
                        layerstring = char(strcat(layerstring,{' '},num2str(i)));
                    else
                        layerstring = char(strcat(layerstring,{', '},num2str(i)));
                    end
                end
            end
            
            
            switch options.D
                case 0
                    XAXISL = {'Relative Modulus of';layerstring};
                case 1
                    XAXISL = {'Modulus [Pa] of';layerstring};
            end
        else
            XIND = find(TMatrix(:,2)-TMatrix(:,1)>0,1,'first');
            XAXIS = TMatrix(XIND,:);
            KAXIS = Kappa(1,:)/scaleK;
            FAXIS = Force(1,:)/scaleF;
            WAXIS = Work(1,:)/scaleW;
            layerstring = {'Layers '};
            for i = 1:NLayers
                if options.LV(i) == 1
                    if i == XIND
                        layerstring = char(strcat(layerstring,{' '},num2str(i)));
                    else
                        layerstring = char(strcat(layerstring,{', '},num2str(i)));
                    end
                end
            end
            
            switch options.D
                case 0
                    XAXISL = {'Relative Thickness of';layerstring};
                case 1
                    XAXISL = {'Thickness [m] of';layerstring};
            end
        end
end
%% PLOTTING KAPPA %%
% -------%
% Plot the 3D surface figure of the curvature.
figure('Position',[30 scrsz(4)/2-120 scrsz(3)/2 scrsz(4)/2],'Name','Kappa')

hold on    % This makes everything appear on the same plot.
switch options.dimen
    case 3
        surf(Ev,Tv,Kappa'/scaleK,'FaceColor','white'); % Plot the surface.
        % Here the 'a' values correspond to the rows of Kappa and 'b'
        % values correspond to the columns of Kappa.
        
        set(gca,'Xscale','log','YScale','log')          % Transform the axes to log scale.
        
        title(strcat(options.ttag,{': Kappa'}),'FontSize',14)   % Create title from ttag
        
        xlim([min(Ev) max(Ev)])     % Match the limits to the variable extremes
        ylim([min(Tv) max(Tv)])     % Match the limits to the variable extremes
        
        set(gca,'FontSize',14)      % Set axis font size
        
        % Set X, Y and Z axis accordingly
        switch options.D
            case 0
                xlabel({'Relative Moduli of';layerstring},'FontSize',14)
                ylabel({'Relative Thickness of';layerstring},'FontSize',14)
                zlabel('Curvature (Kappa)','FontSize',14)
            case 1
                xlabel({'Moduli [Pa] of';layerstring},'FontSize',14)
                ylabel({'Thickness [m] of';layerstring},'FontSize',14)
                zlabel('Curvature (Kappa) [1/m]','FontSize',14)
        end
        
        grid on
        set(gcf, 'color', 'white')  % Sets the background color.
        
        % Set Camera angle
        az = -125;
        el = 40;
        view(az, el);
        
    case 2
        plot(XAXIS,KAXIS);
        set(gca,'XScale','log')
        title(strcat(options.ttag,{': Kappa'}),'FontSize',14)   % Create title from ttag
        
        xlim([min(XAXIS) max(XAXIS)])     % Match the limits to the variable extremes
        set(gca,'FontSize',14)      % Set axis font size
        
        % Set X, Y and Z axis accordingly
        switch options.D
            case 0
                ylabel('Curvature (Kappa)','FontSize',14)
            case 1
                ylabel('Curvature (Kappa) [1/m]','FontSize',14)
        end
        xlabel(XAXISL,'FontSize',14)
        grid on
        set(gcf, 'color', 'white')  % Sets the background color.
end

%% PLOTTING FORCE %%
% -------%
% Plot the 3D surface figure of the force.
figure('Position',[50 scrsz(4)/2-140 scrsz(3)/2 scrsz(4)/2],'Name','Blocked Force')

hold on    % This makes everything appear on the same plot.
switch options.dimen
    case 3
        surf(Ev,Tv,Force'/scaleF,'FaceColor','white'); % Plot the surface.
        % Here the 'a' values correspond to the rows of Kappa and 'b'
        % values correspond to the columns of Kappa.
        
        set(gca,'Xscale','log','YScale','log')          % Transform the axes to log scale.
        
        title(strcat(options.ttag,{': Blocked Force'}),'FontSize',14)   % Create title from ttag
        
        xlim([min(Ev) max(Ev)])     % Match the limits to the variable extremes
        ylim([min(Tv) max(Tv)])     % Match the limits to the variable extremes
        
        set(gca,'FontSize',14)      % Set axis font size
        
        % Set X, Y and Z axis accordingly
        switch options.D
            case 0
                xlabel({'Relative Moduli of';layerstring},'FontSize',14)
                ylabel({'Relative Thickness of';layerstring},'FontSize',14)
                zlabel('Blocked Force','FontSize',14)
            case 1
                xlabel({'Moduli [Pa] of';layerstring},'FontSize',14)
                ylabel({'Thickness [m] of';layerstring},'FontSize',14)
                zlabel('Blocked Force [N]','FontSize',14)
        end
        
        grid on
        set(gcf, 'color', 'white')  % Sets the background color.
        
        % Set Camera angle
        az = -125;
        el = 40;
        view(az, el);
        
    case 2
        plot(XAXIS,FAXIS);
        set(gca,'XScale','log')
        title(strcat(options.ttag,{': Blocked Force'}),'FontSize',14)   % Create title from ttag
        
        xlim([min(XAXIS) max(XAXIS)])     % Match the limits to the variable extremes
        set(gca,'FontSize',14)      % Set axis font size
        
        % Set X, Y and Z axis accordingly
        switch options.D
            case 0
                ylabel('Blocked Force','FontSize',14)
            case 1
                ylabel('Blocked Force [N]','FontSize',14)
        end
        xlabel(XAXISL,'FontSize',14)
        grid on
        set(gcf, 'color', 'white')  % Sets the background color.
end

%% PLOTTING WORK %%
% -------%
% Plot the 3D surface figure of the Work.
figure('Position',[70 scrsz(4)/2-160 scrsz(3)/2 scrsz(4)/2],'Name','Work')

hold on    % This makes everything appear on the same plot.
switch options.dimen
    case 3
        surf(Ev,Tv,Work'/scaleW,'FaceColor','white'); % Plot the surface.
        % Here the 'a' values correspond to the rows of Kappa and 'b'
        % values correspond to the columns of Kappa.
        
        set(gca,'Xscale','log','YScale','log')          % Transform the axes to log scale.
        
        title(strcat(options.ttag,{': Work'}),'FontSize',14)   % Create title from ttag
        
        xlim([min(Ev) max(Ev)])     % Match the limits to the variable extremes
        ylim([min(Tv) max(Tv)])     % Match the limits to the variable extremes
        
        set(gca,'FontSize',14)      % Set axis font size
        
        switch options.D
            case 0
                xlabel({'Relative Moduli of';layerstring},'FontSize',14)
                ylabel({'Relative Thickness of';layerstring},'FontSize',14)
                zlabel('Work','FontSize',14)
            case 1
                xlabel({'Moduli [Pa] of';layerstring},'FontSize',14)
                ylabel({'Thickness [m] of';layerstring},'FontSize',14)
                zlabel('Work [J]','FontSize',14)
        end
        
        grid on
        set(gcf, 'color', 'white')  % Sets the background color.
        
        % Set Camera angle
        az = -125;
        el = 40;
        view(az, el);
        
    case 2
        plot(XAXIS,WAXIS);
        set(gca,'XScale','log')
        title(strcat(options.ttag,{': Work'}),'FontSize',14)   % Create title from ttag
        
        xlim([min(XAXIS) max(XAXIS)])     % Match the limits to the variable extremes
        set(gca,'FontSize',14)      % Set axis font size
        
        % Set X, Y and Z axis accordingly
        switch options.D
            case 0
                ylabel('Work','FontSize',14)
            case 1
                ylabel('Work [J]','FontSize',14)
        end
        xlabel(XAXISL,'FontSize',14)
        grid on
        set(gcf, 'color', 'white')  % Sets the background color.
end

